#!/bin/busybox sh

strip_spaces() (
    STRIP_RE='s/^[[:space:]]*\([^[:space:]].*[^[:space:]]\)[[:space:]]*$/\1/g'
    printf "%s" "$1" | sed -e "$STRIP_RE"
)

matches_string() {
    case "$2" in
        $1) return 0;;
        *) return 1;;
    esac
}

verify_array()
{
    PATTERN="${1#ARRAY:}"

    local SIZE="$(eval echo "\$${PATTERN}_COUNT")"

    if [ -z $SIZE ] ; then
        printf "ERROR: missing element count of array %s in file:\n  %s\n"\
            $PATTERN "$FILENAME"
        return 1
    fi

    # http://stackoverflow.com/questions/2210349/bash-test-whether-string-is-valid-as-an-integer
    [ $SIZE -eq $SIZE ] 2> /dev/null || {
        printf "ERROR: invalid element count of array %s in file:\n  %s\n"\
            $PATTERN "$FILENAME"
        return 1
    }

    for i in $(seq $((SIZE - 1))); do
        VALUE="$(eval echo "\$${PATTERN}_$i")"
        if [ -z $VALUE ]; then
            printf "ERROR: elements missing in array %s in file:\n  %s\n"\
                "${v#ARRAY:}" "$FILENAME"
            return 1
        fi
    done
}

parse_configuration() {
    FILENAME=$1
    shift
    VARIABLES="$@"

    if [ ! -r "$FILENAME" ]; then
        echo "Can not read $FILENAME"
        return 1
    fi

    if [ "${VERBOSITY:-0}" -gt 0 ]; then
        echo "  Parsing $FILENAME";
    fi

    for v in $VARIABLES; do
        if matches_string "ARRAY:*" $v; then
            continue
        fi
        eval "$v="
    done

    while IFS="" read config_line; do
        # Ignore comments and empty lines, and check the line format
        CLEAN_LINE="${config_line%%#*}"
        CLEAN_LINE="$(strip_spaces "$CLEAN_LINE")"
        if [ -z "$CLEAN_LINE" ]; then
            continue
        fi

        if ! matches_string "*=*" "$CLEAN_LINE"; then
            printf "ERROR: could not parse line:\n%s\n" "$CLEAN_LINE"
            return 1
        fi

        # Extract the variable name and the value.

        VARIABLE_NAME="$(strip_spaces "${CLEAN_LINE%%=*}")"
        VALUE="$(strip_spaces "${CLEAN_LINE#*=}")"

        # Verify if the variable in the file is one of the variables being
        # parsed, and set it. Skip variables not requested in VARIABLES

        for v in $VARIABLES; do
            if [ "$VARIABLE_NAME" = "$v" ]; then
                # TODO: CHECK FOR PREVIOUS VALUE (DUPLICATED VARIABLES)
                eval "$v=\"$VALUE\""
            elif [ "$VARIABLE_NAME" = ${v#ARRAY:}_COUNT ] ||
                matches_string ${v#ARRAY:}_[0-9]* $VARIABLE_NAME; then
                # allow patterns in variable names
                eval "$VARIABLE_NAME=\"$VALUE\""
            fi
        done

    done < "$FILENAME"

    if [ "${VERBOSITY:-0}" -gt 0 ]; then
        for v in $VARIABLES; do
            VALUE="$(eval "printf \"%s\" \"\$$v\"")"
            printf "    %s = %s\n" "$v" "$VALUE"
        done
    fi

    UNSET_VARIABLE=
    for v in $VARIABLES; do
        if matches_string "ARRAY:*" $v; then
            if ! verify_array $v; then
                UNSET_VARIABLE=1
            fi
        else
            VALUE="$(eval "printf \"%s\" \"\$$v\"")"
            if [ -z "$VALUE" ]; then
                UNSET_VARIABLE=1
                printf "ERROR: required variable %s not set in file:\n  %s\n"\
                    "$v" "$FILENAME"
            fi
        fi
    done

    if [ -n "$UNSET_VARIABLE" ]; then
        return 1
    fi

    return 0
}
