#!/bin/sh

BACKUP_DIR="$SDO_ROOT_DIR"/media/card/backup

date_iso8601() {
    "$SDO_ROOT_DIR"/bin/date +"%FT%T%z"
}

parse_date_only()
{
    # busybox refuses to parse date only also requires a timestamp
    "$SDO_ROOT_DIR"/bin/date -d "$1 12:00:00" +"%s"
}

ls_backups()
{
    for bak in "$BACKUP_DIR"/*; do
        if [ ! -d "$bak" ]; then
            continue
        fi
        dat="$(basename "$bak")"
        if ! parse_date_only $dat > /dev/null 2>&1; then
            echo "unexpected backup folder: $bak" >&2
            continue
        fi
        echo $dat
    done
}

remove_backup() {
    echo remove $bak
    "$SDO_ROOT_DIR"/bin/rm -rf $BACKUP_DIR/$bak
}

rm_older_2years()
{
    YEARS="$(expr $("$SDO_ROOT_DIR"/bin/date "+%Y") - 2)"
    MONTH_DAY="$("$SDO_ROOT_DIR"/bin/date "+%m-%d")"
    THRESHOLD_DATE="${YEARS}-${MONTH_DAY}"
    THRESHOLD_SEC=$(parse_date_only $THRESHOLD_DATE)

    # convert it back full cycle
    echo "threshold date: $(date -d @${THRESHOLD_SEC} +"%F")"

    for bak in $(ls_backups); do
        SEC=$(parse_date_only "$bak")
        if [ $SEC -lt $THRESHOLD_SEC ]; then
            remove_backup $bak
            continue
        fi
        echo "keep   $bak"
    done
}


if [ -n "$UNITTEST" ]; then
    return
fi

echo "$(date_iso8601) Starting discard old backups"
rm_older_2years
