#!/bin/sh

set -e
# set -xe

. ./test-functions.sh
. ./cleanup-handler.sh

UNITTEST=1
. ./sdo-mount.sh


echo "######################################"
echo "# TEST: isnum                        #"
echo "######################################"
check "isnum 0" 0
check "isnum 7 foo" 0
check "isnum " 1
check "isnum f" 1
check "isnum 3746477" 0
check "isnum -3746477" 0
check "isnum 3E+7" 1
test_passed


echo "######################################"
echo "# TEST: emit_log                     #"
echo "######################################"
check "emit_log check-ret-code" 0
FOO=0x7f
BAR=0xbeef
check "emit_log ipsum lirum $FOO $BAR" 0 "SD-CARD: ipsum lirum 0x7f 0xbeef"
test_passed

echo "
  DESTRUCTIVE TESTS

  The following tests will destroy the filesystem and partition
  table on the SD-card and verify sdo-mount can correctly restore them
"
echo -n "Continue? y/N "
read answer

if [ -z "$answer" -o "$answer" != 'y' ]; then
    echo "abort..."
    exit
fi

if ! type partx >> /dev/null ; then
    die "This test requires 'util-linux-partx', please install"
fi

cleanup_push "/etc/init.d/runit.sh start"
cleanup_push "/etc/init.d/syslog start"
cleanup_push "/usr/sbin/sdo-mount.sh"
umount_card()
{
    /etc/init.d/runit.sh stop || true
    /etc/init.d/syslog stop || true
    sv kill dss || true # usual suspect
    sleep 7
    umount /var/log || true
    umount /usr/share/dss/data/metering || true
    umount /media/card || true
}

umount_card
if mountpoint -q /media/card; then
    die "run sdo-umount.sh before starting"
fi

echo > $REBOOT_LOG

echo "######################################"
echo "# TEST: test blkid                   #"
echo "######################################"
# might save us if we run on a laptop with a SD-card inserted
fetch_blkid_desc
check "test $LABEL = DSSDATA" 0
test_passed

echo "######################################"
echo "# TEST: destroy/restore filesystem   #"
echo "######################################"
destroy_filesystem()
{
    dd if=/dev/zero of=/dev/mmcblk0p1 bs=512 count=10 2> /dev/null
}

destroy_filesystem
mount_sd
check "mountpoint -q /media/card" 0
test_passed
umount_card

echo "######################################"
echo "# TEST: destroy fstab                #"
echo "######################################"
destroy_fstab()
{

    # remove partition from kernel
    if [ -b /dev/mmcblk0p1 ]; then
        # TODO need util-linux-partx
        partx -d /dev/mmcblk0p1
    fi
    dd if=/dev/zero of=/dev/mmcblk0 bs=512 count=10 2> /dev/null
}

destroy_fstab
mount_sd
check "mountpoint -q /media/card" 0
test_passed
umount_card


echo "######################################"
echo "# TEST: remove journal               #"
echo "######################################"

# add journal
tune2fs -j /dev/mmcblk0p1 2> /dev/null
check "has_journal" 0

mount_sd # shall remove the journal
check "has_journal" 1

test_passed
umount_card


echo "
  ALL PASSED

  Don't forget to run sdo-mount from cmdline as well. The main()
  function in sdo-mount is not covered by this test. Read: we really
  had bugs in there in the past.

  ./sdo-umount.sh
  ./sdo-mounts.sh

  Restart your platform to make sure all daemons are restarted
"
